CREATE TABLE RM_ENVIRONMENT(
  ID INTEGER NOT NULL ,
  NAME VARCHAR(128) NOT NULL,
  PRIMARY KEY (ID),
  UNIQUE (NAME)
)ENGINE INNODB;

CREATE TABLE RM_SERVER_INSTANCE (
  ID INTEGER NOT NULL ,
  ENVIRONMENT_ID INTEGER NOT NULL,
  NAME VARCHAR(128) NOT NULL,
  SERVER_URL VARCHAR(1024) NOT NULL,
  DBMS_TYPE VARCHAR(128) NOT NULL,
  INSTANCE_TYPE VARCHAR(128) NOT NULL,
  SERVER_CATEGORY VARCHAR(128) NOT NULL,
  ADMIN_USERNAME VARCHAR(128),
  ADMIN_PASSWORD VARCHAR(128),
  TENANT_ID INTEGER NOT NULL,
  UNIQUE (NAME, ENVIRONMENT_ID, TENANT_ID),
  PRIMARY KEY (ID),
  CONSTRAINT FOREIGN KEY (ENVIRONMENT_ID) REFERENCES RM_ENVIRONMENT (ID)  ON DELETE CASCADE ON UPDATE CASCADE
)ENGINE INNODB;

CREATE TABLE RM_DATABASE (
  ID INTEGER NOT NULL ,
  NAME VARCHAR(128) NOT NULL,
  RSS_INSTANCE_ID INTEGER NOT NULL,
  TYPE VARCHAR(15) NOT NULL,
  TENANT_ID INTEGER NOT NULL,
  UNIQUE (NAME, RSS_INSTANCE_ID, TENANT_ID),
  PRIMARY KEY (ID),
  CONSTRAINT FOREIGN KEY (RSS_INSTANCE_ID) REFERENCES RM_SERVER_INSTANCE (ID)  ON DELETE CASCADE ON UPDATE CASCADE
)ENGINE INNODB;

CREATE TABLE RM_DATABASE_USER (
  ID INTEGER NOT NULL ,
  USERNAME VARCHAR(16) NOT NULL,
  ENVIRONMENT_ID INTEGER NOT NULL,
  TYPE VARCHAR(15) NOT NULL,
  TENANT_ID INTEGER NOT NULL,
  UNIQUE (USERNAME, TENANT_ID, ENVIRONMENT_ID),
  PRIMARY KEY (ID)
)ENGINE INNODB;

CREATE TABLE RM_USER_DATABASE_ENTRY (
  ID INTEGER NOT NULL ,
  DATABASE_USER_ID INTEGER NOT NULL,
  DATABASE_ID INTEGER NOT NULL,
  PRIMARY KEY (ID),
  UNIQUE (DATABASE_USER_ID, DATABASE_ID),
  CONSTRAINT FOREIGN KEY (DATABASE_USER_ID) REFERENCES RM_DATABASE_USER (ID)  ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT FOREIGN KEY (DATABASE_ID) REFERENCES RM_DATABASE (ID)  ON DELETE CASCADE ON UPDATE CASCADE
)ENGINE INNODB;

CREATE TABLE RM_USER_DATABASE_PRIVILEGE (
  ID INTEGER NOT NULL ,
  USER_DATABASE_ENTRY_ID INTEGER NOT NULL,
  SELECT_PRIV enum('N','Y') NOT NULL,
  INSERT_PRIV enum('N','Y') NOT NULL,
  UPDATE_PRIV enum('N','Y') NOT NULL,
  DELETE_PRIV enum('N','Y') NOT NULL,
  CREATE_PRIV enum('N','Y') NOT NULL,
  DROP_PRIV enum('N','Y') NOT NULL,
  GRANT_PRIV enum('N','Y') NOT NULL,
  REFERENCES_PRIV enum('N','Y') NOT NULL,
  INDEX_PRIV enum('N','Y') NOT NULL,
  ALTER_PRIV enum('N','Y') NOT NULL,
  CREATE_TMP_TABLE_PRIV enum('N','Y') NOT NULL,
  LOCK_TABLES_PRIV enum('N','Y') NOT NULL,
  CREATE_VIEW_PRIV enum('N','Y') NOT NULL,
  SHOW_VIEW_PRIV enum('N','Y') NOT NULL,
  CREATE_ROUTINE_PRIV enum('N','Y') NOT NULL,
  ALTER_ROUTINE_PRIV enum('N','Y') NOT NULL,
  EXECUTE_PRIV enum('N','Y') NOT NULL,
  EVENT_PRIV enum('N','Y') NOT NULL,
  TRIGGER_PRIV enum('N','Y') NOT NULL,
  PRIMARY KEY (ID),
  UNIQUE (USER_DATABASE_ENTRY_ID),
  CONSTRAINT FOREIGN KEY (USER_DATABASE_ENTRY_ID) REFERENCES RM_USER_DATABASE_ENTRY (ID)  ON DELETE CASCADE ON UPDATE CASCADE
)ENGINE INNODB;

CREATE TABLE RM_SYSTEM_DATABASE_COUNT (
  ID INTEGER NOT NULL,
  ENVIRONMENT_ID INTEGER NOT NULL,
  COUNT INTEGER NOT NULL DEFAULT 0,
  PRIMARY KEY (ID),
  FOREIGN KEY (ENVIRONMENT_ID) REFERENCES RM_ENVIRONMENT (ID)
)ENGINE INNODB;

CREATE TABLE RM_DB_PRIVILEGE_TEMPLATE (
  ID INTEGER NOT NULL ,
  ENVIRONMENT_ID INTEGER NOT NULL,
  NAME VARCHAR(128) NOT NULL,
  TENANT_ID INTEGER NOT NULL,
  PRIMARY KEY (ID),
  UNIQUE (ENVIRONMENT_ID, NAME, TENANT_ID),
  CONSTRAINT FOREIGN KEY (ENVIRONMENT_ID) REFERENCES RM_ENVIRONMENT (ID)  ON DELETE CASCADE ON UPDATE CASCADE
)ENGINE INNODB;

CREATE TABLE RM_DB_PRIVILEGE_TEMPLATE_ENTRY (
  ID INTEGER NOT NULL ,
  TEMPLATE_ID INTEGER NOT NULL,
  SELECT_PRIV enum('N','Y') NOT NULL,
  INSERT_PRIV enum('N','Y') NOT NULL,
  UPDATE_PRIV enum('N','Y') NOT NULL,
  DELETE_PRIV enum('N','Y') NOT NULL,
  CREATE_PRIV enum('N','Y') NOT NULL,
  DROP_PRIV enum('N','Y') NOT NULL,
  GRANT_PRIV enum('N','Y') NOT NULL,
  REFERENCES_PRIV enum('N','Y') NOT NULL,
  INDEX_PRIV enum('N','Y') NOT NULL,
  ALTER_PRIV enum('N','Y') NOT NULL,
  CREATE_TMP_TABLE_PRIV enum('N','Y') NOT NULL,
  LOCK_TABLES_PRIV enum('N','Y') NOT NULL,
  CREATE_VIEW_PRIV enum('N','Y') NOT NULL,
  SHOW_VIEW_PRIV enum('N','Y') NOT NULL,
  CREATE_ROUTINE_PRIV enum('N','Y') NOT NULL,
  ALTER_ROUTINE_PRIV enum('N','Y') NOT NULL,
  EXECUTE_PRIV enum('N','Y') NOT NULL,
  EVENT_PRIV enum('N','Y') NOT NULL,
  TRIGGER_PRIV enum('N','Y') NOT NULL,
  PRIMARY KEY (ID),
  CONSTRAINT FOREIGN KEY (TEMPLATE_ID) REFERENCES RM_DB_PRIVILEGE_TEMPLATE (ID)  ON DELETE CASCADE ON UPDATE CASCADE
)ENGINE INNODB;

CREATE TABLE RM_USER_INSTANCE_ENTRY (
 RSS_INSTANCE_ID INTEGER NOT NULL ,
 DATABASE_USER_ID INTEGER NOT NULL ,
 PRIMARY KEY (RSS_INSTANCE_ID,DATABASE_USER_ID) ,
 INDEX  (DATABASE_USER_ID ) ,
 INDEX (RSS_INSTANCE_ID ) ,
 CONSTRAINT  FOREIGN KEY (DATABASE_USER_ID) REFERENCES RM_DATABASE_USER (ID) ON DELETE CASCADE ON UPDATE CASCADE,
 CONSTRAINT  FOREIGN KEY (RSS_INSTANCE_ID ) REFERENCES RM_SERVER_INSTANCE (ID) ON DELETE CASCADE ON UPDATE CASCADE
)ENGINE = InnoDB;




